;*****************************************************************************
; Ce sous-programme est destin  tre utilis sur la Maquette  PIC16F877.
;
; Fonction : gestion de l'affichage sur un LCD de 2 lignes de 24 caractres
; 
; Les 3 bits LSB du Port C et le port B est utilis pour l'affichage et le clavier
;
;*****************************************************************************
; Dclarations LCD
;*****************************************************************************
;LCD_DATA	EQU	PORTB		; Lignes de donnes du LCD
;LCD_DATA_TRIS	EQU	TRISB
;LCD_CTRL	EQU	PORTC		; Lignes de Contrle du LCD
;
;LCD_LINE0	EQU	0x000
;LCD_LINE1	EQU	0x040
;
; PORTC bits
;LCD_E		EQU	0		; LCD Enable control line
;LCD_RW		EQU	1		; LCD Read/Write control line
;LCD_RS		EQU	2		; LCD Register-Select control line
;
; misc.
;LCD_TEMP	RES	1		; LCD (variable temporaire)
;LCD_TEMP2	RES	1		; LCD (variable temporaire bis)
;*****************************************************************************
; Routines :
;	LCDINIT	     : initialisation du module LCD
;	LCDBUSY      : on garde la main tant que le module LCD est occup
;	LCDCLEAR     : Effaage de l'cran LCD
;	LCDHOME      : Retour  gauche
;	LCDEMODE(W)  : mise en mode Entry (W)
;	LCDDMODE(W)  : mise en mode DISPLAY (W)
;	LCDPUTCHAR(W) : Ecriture d'un caratre
;	LCDPUTCMD(W) : Ecriture d'une commande	
;*****************************************************************************
; Programme	LCD.ASM
; mise  jour	31/03/00
; Auteur	WEISS Jacques
;
;*****************************************************************************
;*****************************************************************************
; Sous-Programmes
;*****************************************************************************
;*****************************************************************************
;
;=============================================================================
; LCDINIT
; Initialisation du module LCD
;=============================================================================
LCDINIT
					
		CLRF	LCD_CTRL	; Lignes de contrle  '0'
					
		MOVLW	0x01E		; tempo
		CALL	X_DELAY500	; 30 * 0.5mS = 15mS

					
		MOVLW	0x038		; interface 8 bits, 2-lignes
		CALL	LCDPUTCMD
		MOVLW	0x000		; disp.off, curs.off, no-blink
		CALL	LCDDMODE
		CALL	LCDCLEAR
		MOVLW	0x004		; disp.on, curs.off
		CALL	LCDDMODE
		MOVLW	0x002		; auto-inc (shift-cursor)
		CALL	LCDEMODE
		RETURN
;=============================================================================
; LCD_ENABLE
; Envoi d'une impulsion sur la ligne de contrle LCD_Enable
;=============================================================================
LCD_ENABLE
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=============================================================================
; LCDBUSY
; Attente de libration du module LCD (busy-flag  '0')
;=============================================================================
LCDBUSY
		BANKSEL	TRISB			; Passage au banc 1
		MOVLW	0x0FF			; PORT B en entre
		MOVWF	LCD_DATA_TRIS
		BANKSEL	PORTB			; Retour au banc 0

		BCF	LCD_CTRL, LCD_RS	; Module LCD en mode "commande"
		BSF	LCD_CTRL, LCD_RW	; Opration de lecture

		BSF	LCD_CTRL, LCD_E		; LCD Enabled
		MOVF	LCD_DATA, W		; Lecture : busy flag + DDram @
		BCF	LCD_CTRL, LCD_E		; LCD Disabled

		ANDLW	0x7F			; Extraction du compteur d'@ DDram
		XORLW	0x7F			; si 'FF', alors, il n'y a pas de LCD
		SKPNZ				; on sort alors du sous-programme
		GOTO	LCDNOTBUSY

		BSF	LCD_CTRL, LCD_E		; LCD Enabled (bis)
		MOVF	LCD_DATA, W		; Lecture : busy flag + DDram @
		BCF	LCD_CTRL, LCD_E		; LCD Disabled		

		ANDLW	0x80			; Vrification : Busy flag, '1' = Busy
		SKPZ
		GOTO	LCDBUSY
LCDNOTBUSY	
		BCF	LCD_CTRL, LCD_RW	; orientation en criture
		BANKSEL	TRISA			; Passage au banc 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS		; PORT B en sortie
		BANKSEL	PORTA			; Retour au banc 0
		RETURN
;=============================================================================
; LCDCLEAR
; Effaage du LCD et retour du curseur  l'origine
;=============================================================================
LCDCLEAR
		MOVLW	0x001
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDHOME
; Retour du curseur  l'origine
;=============================================================================
LCDHOME
		MOVLW	0x002
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDEMODE
; Place le module LCD en mode "Entry"
; Le mode doit tre plac dans W :
;  b0	: 0 = pas de dcalage de l'affichage	1 = dcalage de l'affichage
;  b1	: 0 = auto-dcrmentation		1 = auto-incrmentation
;  b2-7	: x
;=============================================================================
LCDEMODE
		ANDLW	0x003		; limination des bits de poids fort
		IORLW	0x004		; dfinition de la fonction
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDDMODE
; Contrle de l'affichage
; Le mode doit tre dfini dans W
;  b0	: 0 = cursor blink off	1 = cursor blink on
;  b1	: 0 = cursor off	1 = cursor on
;  b2	: 0 = display off	1 = display on (display data remains in DDRAM)
;  b3-7	: x
; OK
;=============================================================================
LCDDMODE
		ANDLW	0x007		; limination des bits de poids fort
		IORLW	0x008		; dfinition de la fonction
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDSCGA
; Dfinition de l'adresse Character-Generator-RAM 
; les donnes CGRAM sont lues aprs cette opration
; L'adresse CGRAM doit se trouver dans W :
;  b0-5	: required CGRAM address
;  b6-7	: don't care
;=============================================================================
LCDSCGA
		ANDLW	0x03F		; limination des bits de poids fort
		IORLW	0x040		; dfinition de la fonction
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDSDDA
; Dfinition de l'adresse Display-Data-RAM 
; les donnes DDRAM sont lues aprs cette opration
; L'adresse DDRAM doit se trouver dans W :
;  b0-6	: adresse DDRAM 
;  b7	: x
;=============================================================================
LCDSDDA
		IORLW	0x080		; dfinition de la fonction
		CALL	LCDPUTCMD
		RETURN
;=============================================================================
; LCDGADDR
; Renvoi la valeur du compteur d'adresses (utilis par DDRAM et CGRAM)
; L'adresse est renvoye dans W
;=============================================================================
LCDGADDR
		BANKSEL	TRISB			; Passage au banc 1
		MOVLW	0x0FF			; PORT B en entre
		MOVWF	LCD_DATA_TRIS
		BANKSEL	PORTB			; Retour au banc 0

		BCF	LCD_CTRL, LCD_RS	; Module LCD en mode "commande"
		BSF	LCD_CTRL, LCD_RW	; Setup to read busy flag
		BSF	LCD_CTRL, LCD_E		; LCD Enabled
		MOVF	LCD_DATA, W		; Read busy flag + RAM address
		BCF	LCD_CTRL, LCD_E		; LCD Disabled
		ANDLW	0x07F			; Strip upper bit
		BCF	LCD_CTRL, LCD_RW
		BANKSEL	TRISB			; Passage au banc 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS		; PORT B en sortie
		BANKSEL	PORTB			; Retour au banc 0
		RETURN
;=============================================================================
; LCDPUTCHAR
; Affichage d'un caractre sur le LCD
; Le caractre doit se trouver dans W
;=============================================================================
LCDPUTCHAR
		MOVWF	LCD_TEMP		; Stockage du caractre
		MOVWF	TXREG			; envoi sur la ligne RS232

		MOVLW	0x05
		CALL	X_DELAY500	 

		CALL	LCDBUSY			; Attente de disponibilit du module LCD
		BCF	LCD_CTRL, LCD_RW	; Opration de lecture
		BSF	LCD_CTRL, LCD_RS	; Module LCD en mode "data"
		BSF	LCD_CTRL, LCD_E		; LCD Enabled
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA		; Envoi de la donne au module LCD
		BCF	LCD_CTRL, LCD_E		; LCD Disabled
		RETURN
;=============================================================================
; LCDPUTCMD
; Envoi d'une commande au module LCD
; La commande doit se trouver dans W
;=============================================================================
LCDPUTCMD
		MOVWF	LCD_TEMP		; stockage de la commande
		CALL	LCDBUSY			; Attente de disponibilit du module LCD
		BCF	LCD_CTRL, LCD_RW	; Opration de lecture
		BCF	LCD_CTRL, LCD_RS	; Module LCD en mode "commande"
		BSF	LCD_CTRL, LCD_E		; LCD Enabled
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA		; envoi de la donne au module LCD
		BCF	LCD_CTRL, LCD_E		; LCD Disabled
		RETURN



