;*****************************************************************************
; Ce programme est destin  tester la Maquette  PIC16F877.
;
; On stimule tous les ports, sauf le port B et les 3 bits LSB du Port C 
; pour mesure  l'oscilloscope
; 
; Les 3 bits LSB du Port C et le port B est utilis pour l'affichage et le clavier
;
; Programme	TEST877B.ASM
; mise  jour	31/03/00
; Auteur	WEISS Jacques
;
;*****************************************************************************
;*****************************************************************************
	include TEST877b.inc

;*****************************************************************************
; Dpart du Programme
;*****************************************************************************
	ORG	RESET_V		; Vecteur de RESET
RESET		GOTO	START

	ORG	0x100		; les sous-programmes sont en page suprieure
	include LCD.ASM
	include DELAY.ASM
	include KBD.asm	
;*****************************************************************************
; Gestion des interruptions (non gres par ce programme)
;*****************************************************************************
	ORG	ISR_V			; Vecteur d'Interruption
INTER	
	BANKSEL	PORTA		; Select bank 0
	GOTO    INTER


;*****************************************************************************
; Initialize processor registers
;*****************************************************************************
		ORG	0x10
START					; POWER_ON Reset (Beginning of program)

		CLRF	STATUS		; Do initialization, Select bank 0
		CLRF	INTCON		; Clear int-flags, Disable interrupts
		CLRF	PCLATH		; Keep in lower 2KByte

		CLRF	PORTC		; ALL PORT output should output Low.
		CLRF	PORTB

		BANKSEL TRISC		; Selectionne le banc 1
		MOVLW	0x00
		MOVWF	TRISB		; PORT B en SORTIE
		MOVWF	TRISD		; PORT D en SORTIE
		MOVWF	TRISE		; PORT E en SORTIE

		BCF	OPTION_REG, NOT_RBPU	; Disable PORTB pull-ups

;================= Initialisation de la ligne RS232 ===========================

		MOVLW	0x80
		MOVWF	TRISC		; PORT C en SORTIE sauf RC7 (RxD)
		MOVLW	0x10
		MOVWF	TRISA		; PORT A en SORTIE sauf RA4 (RETOUR)

		BSF	TXSTA,BRGH	; High Baud Rate
		BCF	TXSTA,SYNC	; Transmit enable
		BSF	TXSTA,TXEN	; Transmit enable
		MOVLW   D'24'
		MOVWF	SPBRG		; Baud Rate (9600 pour XTAL=4 MHz)

		BANKSEL	PORTA		; Selectionne le banc 0
		
		BSF	RCSTA,SPEN

;============== Fin d'initialisation de la ligne RS232 ========================

		CALL	LCDINIT		; initialisation du module LCD

;*****************************************************************************
; Affichage des lignes :
;	------------------------
;	|Maquette  PIC16f877   |
;	|TEST by J. WEISS       |
;	------------------------
; 
;*****************************************************************************
		MOVLW	0x30		; ASCII '0'
		MOVWF	COUNT		; initialisation des variables de comptage
		MOVWF	COMPTEUR
		MOVWF	CNT_TMP
		MOVLW	'x'
		MOVWF	CLAVIER		; initialisation de la variable CLAVIER

		MOVLW	LCD_LINE0
		CALL	LCDSDDA		; curseur au dbut de la 1re ligne
		MOVLW	LCD_LINE0
		CALL	TABLE_MSG	; Affichage du message

		INCF	COUNT, F
		MOVLW	LCD_LINE1
		CALL	LCDSDDA		; curseur au dbut de la 2me ligne
		MOVLW	LCD_LINE1
		CALL	TABLE_MSG

boucle
		MOVLW	LCD_LINE0 + 0x015
		CALL	LCDSDDA		; curseur  la fin de la 1re ligne
		MOVF	COMPTEUR, W
		CALL	LCDPUTCHAR	; Affichage du compteur


		INCF	COUNT, F


		MOVF	COUNT, W
		SUBLW	0x0FF
		ANDLW	0x01F
		SKPNZ
		INCF	COMPTEUR,F

		MOVLW	0x3A
		SUBWF	COMPTEUR,W
		MOVLW	0x30
		SKPNZ
		MOVWF 	COMPTEUR


		MOVLW	LCD_LINE1 + 0x015
		CALL	LCDSDDA		; curseur  la fin de la 2me ligne

		CALL	KEYBOARD	; lecture du clavier
		SKPZ			; on saute si aucune touche n'a t appuye
		MOVWF	CLAVIER
		MOVFW	CLAVIER
		CALL	LCDPUTCHAR	; affichage du caractre frapp


		MOVLW	0x055		; les lignes qui suivent permettent
		MOVWF	PORTA		; d'activer les ports A, D et E
		MOVWF	PORTD		; afin de verifier les connexions
		MOVWF	PORTE
					
		MOVLW	0x001
		CALL	X_DELAY500	

		MOVLW	0x0AA
		MOVWF	PORTA
		MOVWF	PORTD
		MOVWF	PORTE
				
		MOVLW	0x001
		CALL	X_DELAY500	

		MOVLW	0x0CC
		MOVWF	PORTA
		MOVWF	PORTD
		MOVWF	PORTE
				
		MOVLW	0x001
		CALL	X_DELAY500	

		goto boucle
	

;*****************************************************************************
; le programme s'arrte ici (il tourne en boucle)
;*****************************************************************************

;*****************************************************************************
; Envoi d'un message contenu dans une table
; OK
;*****************************************************************************
TABLE_MSG
		CLRF	TABLE_INDEX	; initialisation de l'@ du message
		ANDLW	0x040
		BTFSS	STATUS, Z	; On vrifie le numro de ligne
		GOTO 	DISP_MSG2a
DISP_MSG1
		MOVWF	TABLE_INDEX	; sauvergarde de l'@ du message
		CALL	MSG1
		ANDLW	0x0FF		; test de fin de message
		BTFSC	STATUS, Z	; (le dernier caractre est 0)
		GOTO	TABLE_MSG_END             
		CALL	LCDPUTCHAR	; affichage du caractre
		MOVF	TABLE_INDEX, W	; pointe vers le caractre suivant
		ADDLW	1
		GOTO	DISP_MSG1
DISP_MSG2a	
		MOVLW	0x000
DISP_MSG2
		MOVWF	TABLE_INDEX	; sauvergarde de l'@ du message
		CALL	MSG2
		ANDLW	0x0FF		; test de fin de message
		BTFSC	STATUS, Z	; (le dernier caractre est 0)
		GOTO	TABLE_MSG_END             
		CALL	LCDPUTCHAR	; affichage du caractre
		MOVF	TABLE_INDEX, W	; pointe vers le caractre suivant
		ADDLW	1
		GOTO	DISP_MSG2

TABLE_MSG_END	RETURN

;=============================================================================
; Table du message de la premire ligne
;=============================================================================
MSG1
		addwf	PCL ,F		; saute  la ligne pointe par W
		retlw	'M'
		retlw	'a'
		retlw	'q'
		retlw	'u'
		retlw	'e'
		retlw	't'
		retlw	't'
		retlw	'e'
		retlw	' '
		retlw	'a'
		retlw	' '
		retlw	'P'
		retlw	'I'
		retlw	'C'
		retlw	'1'
		retlw	'6'
		retlw	'f'
		retlw	'8'
		retlw	'7'
		retlw	'7'
MSG1_END
		retlw	0
;
	IF ( (MSG1 & 0x0FF) >= (MSG1_END & 0x0FF) )
		MESSG   "Warning - User Definded: Table 'MSG1' crosses page boundary in computed jump"
	ENDIF

;=============================================================================
; Table du message de la deuxime ligne
;=============================================================================
MSG2
		addwf	PCL ,F		; saute  la ligne pointe par W
		retlw	'T'
		retlw	'E'
		retlw	'S'
		retlw	'T'
		retlw	' '
		retlw	'b'
		retlw	'y'
		retlw	' '
		retlw	'J'
		retlw	'.'
		retlw	' '
		retlw	'W'
		retlw	'E'
		retlw	'I'
		retlw	'S'
		retlw	'S'
		retlw	' '
		retlw	' '
		retlw	' '
		retlw	' '
MSG2_END
		retlw	0
;
	IF ( (MSG2 & 0x0FF) >= (MSG2_END & 0x0FF) )
		MESSG   "Warning - User Definded: Table 'MSG2' crosses page boundary in computed jump"
	ENDIF


	END
